@@ -5,11 +5,14 @@ import json
 import operator
 import inspect
 import random
+import numpy as np
+import yaml
 
 from logger import logger
 from ..check_utils import check_required_keys, check_type_from_list, \
     check_type, check_file_exists, check_issubclass, check_issubclass_from_list
 from ..file_utils import file_exists
+from ..path_utils import get_extension_from_path
 from ..constants.number_constants import jsonable_types, \
     iterable_jsonable_types, noniterable_jsonable_types
 
@@ -52,7 +55,11 @@ class BasicObject(Generic[T]):
         return obj
 
     def copy(self: T) -> T:
-        return type(self)(**self.to_constructor_dict())
+        constructor_dict = self.to_constructor_dict()
+        for key, val in constructor_dict.items():
+            if hasattr(val, 'copy'):
+                constructor_dict[key] = val.copy()
+        return type(self)(**constructor_dict)
 
 class BasicLoadableObject(BasicObject[T]):
     """
@@ -114,20 +121,46 @@ class BasicLoadableObject(BasicObject[T]):
             logger.warning(f'Expected: {constructor_params}')
             logger.warning(f'Extra: {unnecessary_params}')
         check_required_keys(constructor_dict, required_keys=constructor_params)
-        return cls(*constructor_dict.values())
+        return cls(**constructor_dict)
 
     def save_to_path(self: T, save_path: str, overwrite: bool=False):
         if file_exists(save_path) and not overwrite:
             logger.error(f'File already exists at save_path: {save_path}')
             raise Exception
-        json_dict = self.to_dict()
-        json.dump(json_dict, open(save_path, 'w'), indent=2, ensure_ascii=False)
+
+        extension = get_extension_from_path(save_path)
+        if extension == 'json':
+            json.dump(self.to_dict(), open(save_path, 'w'), indent=2, ensure_ascii=False)
+        elif extension == 'yaml':
+            yaml.dump(self.to_dict(), open(save_path, 'w'), allow_unicode=True)
+        else:
+            raise ValueError(
+                f"""
+                Invalid file extension encountered: {extension}
+                save_path: {save_path}
+
+                Please use either a .json or a .yaml extension.
+                """
+            )
     
     @classmethod
     def load_from_path(cls: T, json_path: str) -> T:
         check_file_exists(json_path)
-        json_dict = json.load(open(json_path, 'r'))
-        return cls.from_dict(json_dict)
+        extension = get_extension_from_path(json_path)
+        if extension == 'json':
+            item_dict = json.load(open(json_path, 'r'))
+        elif extension == 'yaml':
+            item_dict = yaml.load(open(json_path, 'r'), Loader=yaml.FullLoader)
+        else:
+            raise ValueError(
+                f"""
+                Invalid file extension encountered: {extension}
+                json_path: {json_path}
+
+                Please use either a .json or a .yaml extension.
+                """
+            )
+        return cls.from_dict(item_dict)
 
     def verify_jsonable(self):
         def verify_obj(obj: Any, working_path: str):
@@ -347,20 +380,74 @@ class BasicLoadableHandler(BasicHandler[H, T]):
         if file_exists(save_path) and not overwrite:
             logger.error(f'File already exists at save_path: {save_path}')
             raise Exception
-        json_data = self.to_dict_list()
-        json.dump(json_data, open(save_path, 'w'), indent=2, ensure_ascii=False)
+
+        extension = get_extension_from_path(save_path)
+        if extension == 'json':
+            json.dump(self.to_dict_list(), open(save_path, 'w'), indent=2, ensure_ascii=False)
+        elif extension == 'yaml':
+            yaml.dump(self.to_dict_list(), open(save_path, 'w'), allow_unicode=True)
+        else:
+            raise ValueError(
+                f"""
+                Invalid file extension encountered: {extension}
+                save_path: {save_path}
+
+                Please use either a .json or a .yaml extension.
+                """
+            )
     
     @classmethod
     def load_from_path(cls: H, json_path: str) -> H:
         check_file_exists(json_path)
-        json_dict = json.load(open(json_path, 'r'))
-        return cls.from_dict_list(json_dict)
+        extension = get_extension_from_path(json_path)
+        if extension == 'json':
+            item_dict_list = json.load(open(json_path, 'r'))
+        elif extension == 'yaml':
+            item_dict_list = yaml.load(open(json_path, 'r'), Loader=yaml.FullLoader)
+        else:
+            raise ValueError(
+                f"""
+                Invalid file extension encountered: {extension}
+                json_path: {json_path}
+
+                Please use either a .json or a .yaml extension.
+                """
+            )
+        return cls.from_dict_list(item_dict_list)
 
     def verify_jsonable(self):
         for item in self:
             assert hasattr(item, 'verify_jsonable')
             item.verify_jsonable()
 
+    def split(self, ratio: List[int], shuffle: bool=True) -> List[H]:
+        assert len(self) % sum(ratio) == 0, f'sum(ratio)={sum(ratio)} does not evenly divide into len(self)={len(self)}'
+        locations = np.cumsum([val*int(len(self)/sum(ratio)) for val in ratio])
+        start_location = None
+        end_location = 0
+        count = 0
+        samples = []
+        if shuffle:
+            self.shuffle()
+        while count < len(locations):
+            start_location = end_location
+            end_location = locations[count]
+            count += 1
+            samples.append(self[start_location:end_location].copy())
+        return samples
+    
+    def get(self, **kwargs) -> H:
+        def condition(obj) -> bool:
+            for key, val in kwargs.items():
+                if not hasattr(obj, key):
+                    return False
+                elif val is None or (val is not None and getattr(obj, key) == val):
+                    pass
+                else:
+                    return False
+            return True
+        return type(self)([obj for obj in self if condition(obj)])
+
 class BasicLoadableIdHandler(BasicLoadableHandler[H, T], BasicHandler[H, T]):
     """
     TODO: Figure out why VSCode shows syntax error unless I explicitly inherit from all levels of nested parent classes.
@@ -386,6 +473,10 @@ class BasicLoadableIdHandler(BasicLoadableHandler[H, T], BasicHandler[H, T]):
         logger.error(f"Possible ids: {id_list}")
         raise Exception
 
+    @property
+    def ids(self) -> List[int]:
+        return [obj.id for obj in self]
+
 class BasicSubclassHandler(Generic[H, T]):
     """
     TODO: Test Functionality
